==> geodatahub-0.1.3-py2.py3-none-any.whl: geodatahub/auth.py <==
from urllib.parse import urlparse, parse_qs
import requests

class GeodatahubAuth():
    def __init__(self):
        # Tokens
        self.access_token = None
        self.id_token = None
        self.refresh_token = None

        # Login constants
        self._region = "eu-west-1"
        self._redirect_url = 'https://www.geodatahub.dk/validate-code.html'
        self._app_id = "15934a8jitnojg4thofvr8d0io"

        # Auth urls
        self._base_url = "https://geodatahub-dev.auth.eu-west-1.amazoncognito.com"
        self.auth_url = "{base_url}/login?response_type=code&client_id={app_id}&redirect_uri={redirect_url}".format(app_id = self._app_id, redirect_url = self._redirect_url, base_url = self._base_url)
        print(self.auth_url)

        self._boto_client = boto3.client('cognito-idp', self._region)

    def login(self, code = None):
        """Authenticate using one-time code

        Users are expected to retrieve an one-time code
        by visiting www.geodatahub.dk/login in their browser
        and going though the login process.

        Parameters
        ------------
        code: str
          One-time code from a OAuth2 authorization code flow

        Returns
        ---------
        access_token: str
          Token that contains claims about the authenticated user
        refresh_token: str
          Token used to reauthenticate the user without password
        id_token: str
          Token that contains claims about the identify of the user

        Raises
        -------
        RuntimeError:
          If the authentication response did not contain the expected
          parameters.
        """
        # Trade code for token
        payload = {
            'grant_type':'authorization_code',
            'client_id':self._app_id,
            'code':code,
            'redirect_uri': self._redirect_url
        }

        auth_url = "{base_url}/oauth2/token".format(base_url = self._base_url)
        headers = {'Content-type': 'application/x-www-form-urlencoded'}
        resp = requests.post(auth_url, data=payload, headers=headers)

        jresp = resp.json()
        try:
            self.access_token = jresp["access_token"]
            self.refresh_token = jresp["refresh_token"]
            self.id_token = jresp["id_token"]
        except KeyError:
            raise RuntimeError("Token response was malformed!")

        return self.access_token, refresh_token, self.id_token

    def refresh_auth(self, refresh_token = None):
        """Re-authenticate with backend

        Parameters
        -----------
        refresh_token: str, optional
          Token from a previous authentication. If no token is given the
          class refresh token will be used if it exists.

        Returns
        ---------
        access_token: str
          Token that contains claims about the authenticated user
        refresh_token: str
          Token used to reauthenticate the user without password
        id_token: str
          Token that contains claims about the identify of the user

        Raises
        --------
        RuntimeError:
          If a non-recoverable event happened. This would be an unexpected
          response from the authentication step.
        """

        if refresh_token is None:
            refresh_token = self.refresh_token

        # The request should be sent directly to this URL
        auth_url = "https://cognito-idp.eu-west-1.amazonaws.com/"
        headers = {
            "X-Amz-Target": "AWSCognitoIdentityProviderService.InitiateAuth",
            "Content-Type": "application/x-amz-json-1.1",
        }
        payload = {
            'ClientId': self._app_id,
            'AuthFlow': 'REFRESH_TOKEN_AUTH',
            'AuthParameters': {
                'REFRESH_TOKEN': refresh_token
            }
        }

        headers = {'Content-type': 'application/x-www-form-urlencoded'}
        resp = requests.post(auth_url, data=payload, headers=headers)

        jresp = resp.json()
        try:
            self.access_token = resp["AuthenticationResult"]["AccessToken"]
            self.id_token = resp["AuthenticationResult"]["IdToken"]
        except KeyError:
            raise RuntimeError("Token response was malformed!")

        return self.access_token, refresh_token, self.id_token

    def _sign_with_sig4(self, method, path):
        # ************* TASK 1: CREATE A CANONICAL REQUEST *************
        # http://docs.aws.amazon.com/general/latest/gr/sigv4-create-canonical-request.html

        # Create a date for headers and the credential string
        t = datetime.datetime.utcnow()
        amzdate = t.strftime('%Y%m%dT%H%M%SZ')
        datestamp = t.strftime('%Y%m%d') # Date w/o time, used in credential scope

        # Step 1 is to define the verb (GET, POST, etc.)--already done.
#        method = "GET"

        # Step 2: Create canonical URI--the part of the URI from domain to query 
        # string (use '/' if no path)
        canonical_uri = path
        
        # Step 3: Create the canonical query string. In this example (a GET request),
        # request parameters are in the query string. Query string values must
        # be URL-encoded (space=%20). The parameters must be sorted by name.
        # For this example, the query string is pre-formatted in the request_parameters variable.
        canonical_querystring = ""#request_parameters
        
        # Step 4: Create the canonical headers and signed headers. Header names
        # must be trimmed and lowercase, and sorted in code point order from
        # low to high. Note that there is a trailing \n.
        canonical_headers = 'host:' + self._host + '\n' + 'x-amz-date:' + amzdate + '\n'
        
        # Step 5: Create the list of signed headers. This lists the headers
        # in the canonical_headers list, delimited with ";" and in alpha order.
        # Note: The request can include any headers; canonical_headers and
        # signed_headers lists those that you want to be included in the 
        # hash of the request. "Host" and "x-amz-date" are always required.
        signed_headers = 'host;x-amz-date'
        
        # Step 6: Create payload hash (hash of the request body content). For GET
        # requests, the payload is an empty string ("").
        payload_hash = hashlib.sha256(('{}').encode('utf-8')).hexdigest()
        
        # Step 7: Combine elements to create canonical request
        canonical_request = method + '\n' + canonical_uri + '\n' + canonical_querystring + '\n' + canonical_headers + '\n' + signed_headers + '\n' + payload_hash

        # ************* TASK 2: CREATE THE STRING TO SIGN*************
        # Match the algorithm to the hashing algorithm you use, either SHA-1 or
        # SHA-256 (recommended)
        algorithm = 'AWS4-HMAC-SHA256'
        credential_scope = datestamp + '/' + self._region + '/' + self._service + '/' + 'aws4_request'
        string_to_sign = algorithm + '\n' +  amzdate + '\n' +  credential_scope + '\n' +  hashlib.sha256(canonical_request.encode('utf-8')).hexdigest()
        
        # ************* TASK 3: CALCULATE THE SIGNATURE *************
        # Create the signing key using the function defined above.
        signing_key = getSignatureKey(self._access_key, datestamp, self._region, self._service)
        
        # Sign the string_to_sign using the signing_key
        signature = hmac.new(signing_key, (string_to_sign).encode('utf-8'), hashlib.sha256).hexdigest()

        # ************* TASK 4: ADD SIGNING INFORMATION TO THE REQUEST *************
        # The signing information can be either in a query string value or in 
        # a header named Authorization. This code shows how to use a header.
        # Create authorization header and add to request headers
        authorization_header = algorithm + ' ' + 'Credential=' + self._access_key + '/' + credential_scope + ', ' +  'SignedHeaders=' + signed_headers + ', ' + 'Signature=' + signature
        
        # The request can include any headers, but MUST include "host", "x-amz-date", 
        # and (for this scenario) "Authorization". "host" and "x-amz-date" must
        # be included in the canonical_headers and signed_headers, as noted
        # earlier. Order here is not significant.
        # Python note: The 'host' header is added automatically by the Python 'requests' library.
        auth_headers = {'x-amz-date':amzdate,
                        'Authorization':authorization_header}

        return auth_headers

# Key derivation functions. See:
# http://docs.aws.amazon.com/general/latest/gr/signature-v4-examples.html#signature-v4-examples-python
def sign(key, msg):
    return hmac.new(key, msg.encode('utf-8'), hashlib.sha256).digest()

def getSignatureKey(key, dateStamp, regionName, serviceName):
    kDate = sign(('AWS4' + key).encode('utf-8'), dateStamp)
    kRegion = sign(kDate, regionName)
    kService = sign(kRegion, serviceName)
    kSigning = sign(kService, 'aws4_request')
    return kSigning

==> geodatahub-0.1.4-py2.py3-none-any.whl: geodatahub/auth.py <==
from urllib.parse import urlparse, parse_qs
import requests

class GeodatahubAuth():
    def __init__(self):
        # Tokens
        self.access_token = None
        self.id_token = None
        self.refresh_token = None

        # Login constants
        self._region = "eu-west-1"
        self._redirect_url = 'https://www.geodatahub.dk/validate-code.html'
#        self._app_id = "15934a8jitnojg4thofvr8d0io"
        # map-dev.geodatahub.dk
        self._app_id = "2rk82qpa68at0uro563rj0ihcf"

        # Auth urls
        self._base_url = "https://geodatahub-dev.auth.eu-west-1.amazoncognito.com"
        self.auth_url = "{base_url}/login?response_type=code&client_id={app_id}&redirect_uri={redirect_url}".format(app_id = self._app_id, redirect_url = self._redirect_url, base_url = self._base_url)

    def login(self, code = None):
        """Authenticate using one-time code

        Users are expected to retrieve an one-time code
        by visiting www.geodatahub.dk/login in their browser
        and going though the login process.

        Parameters
        ------------
        code: str
          One-time code from a OAuth2 authorization code flow

        Returns
        ---------
        access_token: str
          Token that contains claims about the authenticated user
        refresh_token: str
          Token used to reauthenticate the user without password
        id_token: str
          Token that contains claims about the identify of the user

        Raises
        -------
        RuntimeError:
          If the authentication response did not contain the expected
          parameters.
        """
        # Trade code for token
        payload = {
            'grant_type':'authorization_code',
            'client_id':self._app_id,
            'code':code,
            'redirect_uri': self._redirect_url
        }

        auth_url = "{base_url}/oauth2/token".format(base_url = self._base_url)
        headers = {'Content-type': 'application/x-www-form-urlencoded'}
        resp = requests.post(auth_url, data=payload, headers=headers)

        jresp = resp.json()
        try:
            self.access_token = jresp["access_token"]
            self.refresh_token = jresp["refresh_token"]
            self.id_token = jresp["id_token"]
        except KeyError:
            raise RuntimeError("Token response was malformed!")

        return self.access_token, refresh_token, self.id_token

    def refresh_auth(self, refresh_token = None):
        """Re-authenticate with backend

        Parameters
        -----------
        refresh_token: str, optional
          Token from a previous authentication. If no token is given the
          class refresh token will be used if it exists.

        Returns
        ---------
        access_token: str
          Token that contains claims about the authenticated user
        refresh_token: str
          Token used to reauthenticate the user without password
        id_token: str
          Token that contains claims about the identify of the user

        Raises
        --------
        RuntimeError:
          If a non-recoverable event happened. This would be an unexpected
          response from the authentication step.
        """

        if refresh_token is None:
            refresh_token = self.refresh_token

        # The request should be sent directly to this URL
        auth_url = "https://cognito-idp.eu-west-1.amazonaws.com/"
        headers = {
            "X-Amz-Target": "AWSCognitoIdentityProviderService.InitiateAuth",
            "Content-Type": "application/x-amz-json-1.1",
        }
        payload = {
            "ClientId": self._app_id,
            "AuthFlow": "REFRESH_TOKEN_AUTH",
            "AuthParameters": {
                "REFRESH_TOKEN": refresh_token
            }
        }

        resp = requests.post(auth_url, json=payload, headers=headers)

        jresp = resp.json()
        try:
            self.access_token = jresp["AuthenticationResult"]["AccessToken"]
            self.id_token = jresp["AuthenticationResult"]["IdToken"]
        except KeyError:
            raise RuntimeError("Token response was malformed!")

        return self.access_token, refresh_token, self.id_token

    def _sign_with_sig4(self, method, path):
        # ************* TASK 1: CREATE A CANONICAL REQUEST *************
        # http://docs.aws.amazon.com/general/latest/gr/sigv4-create-canonical-request.html

        # Create a date for headers and the credential string
        t = datetime.datetime.utcnow()
        amzdate = t.strftime('%Y%m%dT%H%M%SZ')
        datestamp = t.strftime('%Y%m%d') # Date w/o time, used in credential scope

        # Step 1 is to define the verb (GET, POST, etc.)--already done.
#        method = "GET"

        # Step 2: Create canonical URI--the part of the URI from domain to query 
        # string (use '/' if no path)
        canonical_uri = path
        
        # Step 3: Create the canonical query string. In this example (a GET request),
        # request parameters are in the query string. Query string values must
        # be URL-encoded (space=%20). The parameters must be sorted by name.
        # For this example, the query string is pre-formatted in the request_parameters variable.
        canonical_querystring = ""#request_parameters
        
        # Step 4: Create the canonical headers and signed headers. Header names
        # must be trimmed and lowercase, and sorted in code point order from
        # low to high. Note that there is a trailing \n.
        canonical_headers = 'host:' + self._host + '\n' + 'x-amz-date:' + amzdate + '\n'
        
        # Step 5: Create the list of signed headers. This lists the headers
        # in the canonical_headers list, delimited with ";" and in alpha order.
        # Note: The request can include any headers; canonical_headers and
        # signed_headers lists those that you want to be included in the 
        # hash of the request. "Host" and "x-amz-date" are always required.
        signed_headers = 'host;x-amz-date'
        
        # Step 6: Create payload hash (hash of the request body content). For GET
        # requests, the payload is an empty string ("").
        payload_hash = hashlib.sha256(('{}').encode('utf-8')).hexdigest()
        
        # Step 7: Combine elements to create canonical request
        canonical_request = method + '\n' + canonical_uri + '\n' + canonical_querystring + '\n' + canonical_headers + '\n' + signed_headers + '\n' + payload_hash

        # ************* TASK 2: CREATE THE STRING TO SIGN*************
        # Match the algorithm to the hashing algorithm you use, either SHA-1 or
        # SHA-256 (recommended)
        algorithm = 'AWS4-HMAC-SHA256'
        credential_scope = datestamp + '/' + self._region + '/' + self._service + '/' + 'aws4_request'
        string_to_sign = algorithm + '\n' +  amzdate + '\n' +  credential_scope + '\n' +  hashlib.sha256(canonical_request.encode('utf-8')).hexdigest()
        
        # ************* TASK 3: CALCULATE THE SIGNATURE *************
        # Create the signing key using the function defined above.
        signing_key = getSignatureKey(self._access_key, datestamp, self._region, self._service)
        
        # Sign the string_to_sign using the signing_key
        signature = hmac.new(signing_key, (string_to_sign).encode('utf-8'), hashlib.sha256).hexdigest()

        # ************* TASK 4: ADD SIGNING INFORMATION TO THE REQUEST *************
        # The signing information can be either in a query string value or in 
        # a header named Authorization. This code shows how to use a header.
        # Create authorization header and add to request headers
        authorization_header = algorithm + ' ' + 'Credential=' + self._access_key + '/' + credential_scope + ', ' +  'SignedHeaders=' + signed_headers + ', ' + 'Signature=' + signature
        
        # The request can include any headers, but MUST include "host", "x-amz-date", 
        # and (for this scenario) "Authorization". "host" and "x-amz-date" must
        # be included in the canonical_headers and signed_headers, as noted
        # earlier. Order here is not significant.
        # Python note: The 'host' header is added automatically by the Python 'requests' library.
        auth_headers = {'x-amz-date':amzdate,
                        'Authorization':authorization_header}

        return auth_headers

# Key derivation functions. See:
# http://docs.aws.amazon.com/general/latest/gr/signature-v4-examples.html#signature-v4-examples-python
def sign(key, msg):
    return hmac.new(key, msg.encode('utf-8'), hashlib.sha256).digest()

def getSignatureKey(key, dateStamp, regionName, serviceName):
    kDate = sign(('AWS4' + key).encode('utf-8'), dateStamp)
    kRegion = sign(kDate, regionName)
    kService = sign(kRegion, serviceName)
    kSigning = sign(kService, 'aws4_request')
    return kSigning
